/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ThumbsDown.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


function ThumbnailsPage() // extend WizardPage
{
    var depends = "thumbnails";
    WizardPage.call(this, depends);
    
    this.selected = [];
    this.thumbnailIndex = null;
}

ThumbnailsPage.prototype =
{
    get tree() { return $("tree_thumbnails"); },
    
    get targetCount()
    {
        if (! this._targetCount)
        {
            var count = {image: 0, page: 0, unknown: 0, error: 0, loading: 0};
            for each (var thumbnail in this.selected)
                if (thumbnail.target.head && thumbnail.target.head.result)
                    count[thumbnail.target.head.result]++;
                else
                    count.loading++
            
            if ((count.image + count.page + count.unknown + count.error + count.loading) != this.selected.length)
                throw new Error("target count mismatch");
            
            this._targetCount = count;
        }
        return this._targetCount;
    },
    
    
    onPageShow: function()
    {
        if (this.firstRun)
        {
            var self = this;
            $("treechildren_thumbnails").addEventListener("DOMMouseScroll", function(e) { self.onThumbnailTreeMouseMove(e); }, false);
            $("image_thumbnail").addEventListener("load", function() { self.onThumbnailLoad(); }, false);
            wizard.addObserver(this, TOPIC_TARGET_HEAD, false);
        }
        
        if (this.dirty.thumbnails)
            this.tree.view = new ThumbnailsTreeView(model.thumbnails);
        
        if (this.dirty.page)
            this._updateSelected();
    },
    
    observe: function(subject, topic, data)
    {
        this._targetCount = null;
        
        if (page != pages.thumbnails)
            return;
        
        var row = parseInt(data);
        var col = this.tree.columns["tcol_thumbnails_target"];
        this.tree.treeBoxObject.invalidateCell(row, col);
        this._targetCount = null;
        wizard.updateWizardButtons();
    },
    
    onThumbnailTreeSelect: function()
    {
        this._updateSelected();
        wizard.updateWizardButtons();
    },
    
    onThumbnailTreeKeyUp: function()
    {
        this._setThumbnail(this.tree.currentIndex);
    },
    
    onThumbnailTreeMouseMove: function(e)
    {
        var row = this.tree.treeBoxObject.getRowAt(e.clientX, e.clientY);
        this._setThumbnail((row >= 0) ? row : null);
    },
    
    onThumbnailTreeMouseOut: function(e)
    {
        this._setThumbnail(null);
    },
    
    onThumbnailLoad: function()
    {
        if (page != pages.thumbnails)
            return;
        
        this._updateThumbnailSize();
    },
    
    canAdvance: function()
    {
        if (this.selected.length < 2)
            return false;
        if ((this.targetCount.image == this.selected.length) ||
            (this.targetCount.page == this.selected.length))
            return true;
        return false;
    },
    
    _setThumbnail: function(index)
    {
        if (index == this.thumbnailIndex)
            return;
        
        this.thumbnailIndex = index;
        
        var image = $("image_thumbnail");
        if (index != null)
        {
            var thumbnail = model.thumbnails[index];
            
            var boxComputedStyle = document.defaultView.getComputedStyle($("box_thumbnail"), null);
            var boxWidth = boxComputedStyle.width.replace(/px$/, "");
            var boxHeight = boxComputedStyle.height.replace(/px$/, "");
            
            image.src = thumbnail.image.link;
            image.setSize(boxWidth, boxHeight);
        }
        else
        {
            image.src = "";
            this._updateThumbnailSize();
        }
    },
    
    _updateSelected: function()
    {
        this.selected = [model.thumbnails[row] for each (row in getTreeSelectionIterator(this.tree.view.selection))];
        this._targetCount = null;
        $("lbl_thumbnails_count").value = _("count", this.selected.length, model.thumbnails.length);
    },
    
    _updateThumbnailSize: function()
    {
        var image = $("image_thumbnail");
        if (image.src)
            $("lbl_thumbnailSize").value = _("imageSize", image.naturalWidth, image.naturalHeight);
        else
            $("lbl_thumbnailSize").value = "";
    },
    
    onPageAdvanced: function()
    {
        model.selected = this.selected;
        
        if (! model.matchingScript)
            model.matchingScript = scriptWizard.getMatchingScript(model.selected);
        
        if (model.matchingScript)
            return wizard.advance("script");
        
        if (! model.linkUrlTokens)
            try
            {
                model.linkUrlTokens = scriptWizard.getLinkUrlTokens(model.selected);
                model.error = null;
            }
            catch (e if (typeof(e) == "string"))
            {
                model.error = e;
                return wizard.advance("error");
            }
        
        if (! model.target)
            model.target = ((this.targetCount.image > 0) ? TARGET_IMAGE : TARGET_PAGE);
        
        switch (model.target)
        {
            case TARGET_IMAGE: return wizard.advance("mode");
            case TARGET_PAGE: return wizard.advance("images");
        }
        
        throw new Error("unreachable code");
    }
}

extend(ThumbnailsPage, WizardPage);

pages.thumbnails = new ThumbnailsPage();


function ThumbnailsTreeView(thumbnails) // extend TreeView
{
    TreeView.call(this, thumbnails);
}

ThumbnailsTreeView.prototype =
{
    getCellText: function(row, col)
    {
        var thumbnail = this.model[row];
        switch (col.id)
        {
            case "tcol_thumbnails_link":
                return thumbnail.anchor.link;
            
            case "tcol_thumbnails_target":
                var target = thumbnail.target.head;
                if (! (target && target.result))
                    return _("target.loading");
                switch (target.result)
                {
                    case "image":   return _("target.image");
                    case "page":    return _("target.page");
                    case "unknown": return (target.contentType ? _("target.unknown", (target.mimeType ? target.mimeType : target.contentType)) : _("target.noContentType"));
                    case "error":   return _("target.error", (target.status ? target.status : -1));
                    case "timeout": return _("target.timeout");
                }
                throw new Error("unknown target result: " + target.result);
        }
        throw new Error("unknown column: " + col.id);
    }
}

extend(ThumbnailsTreeView, TreeView);
